@@ -79,7 +79,8 @@ class CreateCluster(BasicCommand):
         {'name': 'no-enable-debugging', 'action': 'store_true',
          'group_name': 'debug'},
         {'name': 'tags', 'nargs': '+',
-         'help_text': helptext.TAGS},
+         'help_text': helptext.TAGS,
+         'schema': argumentschema.TAGS_SCHEMA},
         {'name': 'bootstrap-actions',
          'help_text': helptext.BOOTSTRAP_ACTIONS,
          'schema': argumentschema.BOOTSTRAP_ACTIONS_SCHEMA},
